@@ -40,8 +40,9 @@ def is_valid_name(name: str) -> bool:
 def get_invalid_names() -> List[str]:
     invalid_names = [ 'send', 'rcv', 'len', 'round', 'list', 'filter', 'self', 'jid',
                       'datetime', 'random', 'numpy', 'json', 'spade', 'copy', 'uuid',
-                      'get_json_from_spade_message', 'get_spade_message', 'logger', 'any',
-                      'BackupBehaviour', 'backup_url', 'backup_period', 'backup_delay' ]
+                      'get_json_from_spade_message', 'get_spade_message', 'logger',
+                      'any', 'sys', 'limit_number', 'BackupBehaviour', 'backup_url', 
+                      'backup_period', 'backup_delay' ]
     invalid_names.extend(keyword.kwlist)
     return invalid_names
 
